/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//Copyright (c) 2010 by disy Informationssysteme GmbH
package net.disy.legato.testing.server.tomcat;

import java.io.File;
import java.net.InetAddress;
import java.net.MalformedURLException;

import javax.servlet.ServletContext;

import net.disy.legato.testing.server.IWebServerEnvironment;

import org.apache.catalina.Context;
import org.apache.catalina.Engine;
import org.apache.catalina.Host;
import org.apache.catalina.LifecycleException;
import org.apache.catalina.Wrapper;
import org.apache.catalina.connector.Connector;
import org.apache.catalina.loader.WebappLoader;
import org.apache.catalina.startup.Embedded;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

// NOT_PUBLISHED
public class TomcatWebServerEnvironment implements IWebServerEnvironment {

  private static final Log logger = LogFactory.getLog(TomcatWebServerEnvironment.class);

  private Embedded container = null;
  private Boolean started = false;
  private WebappLoader webAppLoader = null;
  private Host localHost = null;

  /**
   * The classes directory for the web application being run.
   */
  private final String classesDir = "target/classes"; //$NON-NLS-1$

  public TomcatWebServerEnvironment(final String catalinaHome, final int port) {
    container = new Embedded();
    container.setCatalinaHome(catalinaHome);
    //container.setRealm(new MemoryRealm());

    localHost = container.createHost("localHost", new File(".").getAbsolutePath());

    final Engine engine = container.createEngine();
    engine.setName("localEngine");
    engine.addChild(localHost);
    engine.setDefaultHost(localHost.getName());
    container.addEngine(engine);

    final Connector httpConnector = container.createConnector((InetAddress) null, port, false);
    container.addConnector(httpConnector);
    container.setAwait(true);

    webAppLoader = new WebappLoader(this.getClass().getClassLoader());

    if (classesDir != null) {
      try {
        webAppLoader.addRepository(new File(classesDir).toURI().toURL().toString());
      }
      catch (final MalformedURLException e) {
        e.printStackTrace();
      }
    }

  }

  @Override
  public void addServletWithMapping(
      final String contextPath,
      final String servletClassName,
      final String servletMapping) {
    final Context context = getContext(contextPath);
    final Wrapper wrapper = context.createWrapper();
    wrapper.setName(servletClassName);
    wrapper.setServletClass(servletClassName);
    context.addChild(wrapper);
    context.addServletMapping(contextPath + servletMapping, servletClassName);
  }

  @Override
  public void addWebAppContext(final String contextPath, final String location) {
    addWebAppContext(contextPath, location, true);
  }

  @Override
  public void addWebAppContext(
      final String contextPath,
      final String location,
      final boolean cookies) {
    final Context context = container.createContext(contextPath, location);
    context.setLoader(webAppLoader);
    context.setReloadable(true);
    context.setCookies(cookies);
    localHost.addChild(context);
  }

  @Override
  public String getBaseUrl() {
    return "http://" + getHost() + ':' + getPort(); //$NON-NLS-1$
  }

  @Override
  public String getHost() {
    return "localhost"; //$NON-NLS-1$
  }

  @Override
  public int getPort() {
    return container.findConnectors()[0].getPort();
  }

  protected Context getContext(final String contextPath) {
    final Host host = (Host) container.getContainer().findChildren()[0];
    final Context context = host.map(contextPath);
    return context;
  }

  @Override
  public ServletContext getServletContext(final String contextPath) {
    final Context context = getContext(contextPath);
    if (context != null) {
      return context.getServletContext();
    }
    return null;
  }

  @Override
  public boolean isStarted() {
    return started;
  }

  @Override
  public boolean isStopped() {
    return !started;
  }

  @Override
  public void join() throws Exception {
    while (started) {
      Thread.sleep(2000);
    }
  }

  @Override
  public void start() throws Exception {
    synchronized (started) {
      if (started) {
        throw new RuntimeException("Server was already started"); //$NON-NLS-1$
      }
      try {
        container.start();
        started = true;
        // add shutdown hook to stop server
        Runtime.getRuntime().addShutdownHook(new Thread() {
          @Override
          public void run() {
            try {
              TomcatWebServerEnvironment.this.stop();
            }
            catch (final Exception e) {
              logger.error("Could not stop server", e); //$NON-NLS-1$
            }
          }
        });
      }
      catch (final LifecycleException e) {
        logger.error("Could not start server", e); //$NON-NLS-1$
        started = false;
      }
    }

  }

  @Override
  public void stop() throws Exception {
    synchronized (started) {
      if (!started) {
        throw new RuntimeException("Server was not yet started"); //$NON-NLS-1$
      }
      try {
        if (container != null) {
          container.stop();
          started = false;
        }
        else {
          throw new RuntimeException("Container is null"); //$NON-NLS-1$
        }
      }
      catch (final LifecycleException e) {
        logger.error("Could not stop server", e); //$NON-NLS-1$
      }
    }

  }

}
